import xml.etree.ElementTree as Et
import win32evtlog
import sqlite3

# https://stackoverflow.com/questions/43005100/how-to-read-evtx-file-using-python/69320457#69320457

class EventLogParser:
    def __init__(self, exported_log_file):
        self.exported_log_file = exported_log_file

    def get_all_events(self):
        windows_events = []
        query_handle = win32evtlog.EvtQuery(str(self.exported_log_file),
            win32evtlog.EvtQueryFilePath | win32evtlog.EvtQueryReverseDirection)
        while True:
            raw_event_collection = win32evtlog.EvtNext(query_handle, 1)
            if len(raw_event_collection) == 0:
                break
            for raw_event in raw_event_collection:
                windows_events.append(self.parse_raw_event(raw_event))
        return windows_events

    def parse_raw_event(self, raw_event):
        new_event = {}
        new_event_node = {}
        xml_content = win32evtlog.EvtRender(raw_event, win32evtlog.EvtRenderEventXml)

        root = Et.fromstring(xml_content)
        ns = "{" + root.tag.split('}')[0].strip('{') + "}"
        system = root.find(f'{ns}System')
        eventrecord_id = system.find(f'{ns}EventRecordID').text
        channel = system.find(f'{ns}Channel').text.replace("-"," ").replace("/"," ")
        words = channel.split()
        letters = [word[0] for word in words]
        id_prefix = "".join(letters).upper()
        record = id_prefix + eventrecord_id

        try:
            node_system = '<System>'
            terminator_system = "</System>"
            idx1 = xml_content.index(node_system)
            idx_temp = xml_content[idx1 + len(node_system):].index(terminator_system)
            idx2 = idx_temp + idx1 + len(node_system)
            dict_sys_update = xml_content[idx1 + len(node_system): idx2]
            new_event_node.update({'System': dict_sys_update})
        except:
            pass

        try:
            node_eventdata = '<EventData>'
            terminator_eventdata = "</EventData>"
            idx1 = xml_content.index(node_eventdata)
            idx_temp = xml_content[idx1 + len(node_eventdata):].index(terminator_eventdata)
            idx2 = idx_temp + idx1 + len(node_eventdata)
            dict_event_update = xml_content[idx1 + len(node_eventdata): idx2]
            new_event_node.update({'EventData': dict_event_update})
        except:
            pass

        new_event.update({record: new_event_node})
        return new_event

#######################################################################
# DB test  - https://stackoverflow.com/questions/52142645/how-to-improve-sqlite-insert-performance-in-python-3-6
#######################################################################

    def create_db(self):
        conn = sqlite3.connect('events.db')
        c = conn.cursor()
        c.execute('''CREATE TABLE IF NOT EXISTS event (
            recordid TEXT,
            systemdata TEXT,
            eventdata TEXT
            )''')
        conn.commit()
        conn.close()

    def add_db(self, windows_events):
        conn = sqlite3.connect('events.db')
        c = conn.cursor()
        c.execute('''PRAGMA synchronous = EXTRA''')
        c.execute('''PRAGMA journal_mode = WAL''')
        for event_value in windows_events:
            try:
                for key_ev, value_ev in event_value.items():
                    event_values = (key_ev, value_ev['System'], value_ev['EventData'])
                    c.execute("INSERT INTO event VALUES (?,?,?)", event_values)
            except:
                continue
        conn.commit()
        conn.close()


log1 = EventLogParser('security.evtx')
log1.create_db()
windows_evtx = log1.get_all_events()
log1.add_db(windows_evtx)